package mv3302.run;

import mv3302.ArrivalProcess;
import mv3302.SimpleServer;
import simkit.Schedule;
import simkit.random.RandomVariate;
import simkit.random.RandomVariateFactory;
import simkit.stat.SimpleStatsTimeVarying;

/**
 *
 * @author dansl
 */
public class RunSimpleServer {

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        //Creates a random variate and assigns that value to the interarrival time variable
        RandomVariate interarrivalTime = RandomVariateFactory.getInstance("Uniform", 0.9, 2.2);
        //Creates an instance variable of the arrival process
        ArrivalProcess arrivalProcess = new ArrivalProcess(interarrivalTime);
        //Prints out pertinent information about the arrival process in the sim
        System.out.println(arrivalProcess);

        //Creates a random variate with the following properties
        String rvName = "Gamma";
        double alpha = 1.7;
        double beta = 1.8;
        RandomVariate serviceTime = RandomVariateFactory.getInstance(rvName, alpha, beta);

        //Creates a simple server with two servers and varying service times
        SimpleServer simpleServer = new SimpleServer(2, serviceTime);
        //Prints pertinent simple server details
        System.out.println(simpleServer);
        //Adds a sim event listener on the simple server for changes in the arrival process
        arrivalProcess.addSimEventListener(simpleServer);

        //Creates statistics tracking stats of pertinent variable changes in the simulation over time
        SimpleStatsTimeVarying numberInQueueStat = new SimpleStatsTimeVarying("numberInQueue");
        SimpleStatsTimeVarying numberAvailableServersStat = new SimpleStatsTimeVarying("numberAvailableServers");

        //Sets statistics objects as listeners to state changes in the simple server
        simpleServer.addPropertyChangeListener(numberInQueueStat);
        simpleServer.addPropertyChangeListener(numberAvailableServersStat);
        //Display each event & the Event List if value is set to true
        Schedule.setVerbose(false);
        //End at the given simulation time
        Schedule.stopAtTime(100000.0);
        //Invoke reset() on the Simkit component(s)
        Schedule.reset();
        //Execute the simulation
        Schedule.startSimulation();

        //Prints out final simulation statistics by pulling necessary values
        System.out.printf("Simulation ended at time %,.3f%n", Schedule.getSimTime());
        System.out.printf("%nThere have been %d arrivals%n", arrivalProcess.getNumberArrivals());
        System.out.printf("There have been %d customers served%n", simpleServer.getNumberServed());
        System.out.printf("Average number in queue\t%.3f%n", numberInQueueStat.getMean());
        System.out.printf("Average utilization\t%.3f%n", 1.0 - numberAvailableServersStat.getMean() / simpleServer.getTotalNumberServers());
    }
}
